==> reaper-0.001009.tar.gz: reaper-0.001009/lib/Reaper/Node.pm <==
# a reaper item - a process to be terminated at a certain time

package Reaper::Node;

use Moose;
use File::stat;
use DateTime;
use DateTime::TimeZone::Tzfile;
use Unix::PID;
use Return::Value;
use Log::Log4perl qw(get_logger);
use Carp;

has 'file'          => (is => 'ro', required => 1);
has 'time'          => (is => 'ro', required => 1);
has 'tz'            => (is => 'ro', default => sub {
  DateTime::TimeZone::Tzfile->new('/etc/localtime');
});
has 'today'         => (is => 'ro', isa => 'HashRef', auto_deref => 1, lazy => 1, default => sub {
  my $self = shift;
  my $now = DateTime->now(time_zone => $self->tz);
  return { year => $now->year, month => $now->month, day => $now->day, time_zone => $self->tz };
});
has 'date'          => (is => 'ro', lazy => 1, default => sub { 
  my $self = shift;
  my ($hh, $mi) = split /:/, $self->time;
  DateTime->new( $self->today, hour => $hh, minute => $mi );
});
has 'pid'           => (is => 'ro', required => 1);
has 'command'       => (is => 'ro', default => '');
has 'uid'           => (is => 'ro', lazy => 1, default => sub { stat(shift->file)->uid });
has 'log'           => (is => 'ro', lazy => 1, default => sub { get_logger('') });
has 'ps'            => (is => 'ro', lazy => 1, default => sub { 
  Unix::PID->new or die "Cannot instantiate Unix::PID: $!";
});
# Flag for testing to indicate this is a fake process
has 'fake'          => (is => 'ro', isa => 'Bool');
# Flag for running in noop mode - don't really issue kills etc.
has 'noop'          => (is => 'ro', isa => 'Bool');

# Return the number of seconds until 'time'
sub time_seconds
{
  my $self = shift;
  my $now = shift || DateTime->now;
  return 0 if $now >= $self->date;
  return $now->subtract_datetime_absolute( $self->date )->seconds;
}

# Internal version - kill the process associated with this node
sub _kill
{
  my $self = shift;

  # Always return success for fake processes or in noop mode
  return success if $self->fake || $self->noop;

  # Fail if no process has this pid
  my $ps = $self->ps;
  return failure 'ENOPID'
    unless $ps->is_pid_running( $self->pid );

  # Load pid info
  my $pidinfo = $ps->pid_info_hash( $self->pid )
    or die "Cannot get pid info for pid " . $self->pid;

  # Fail if uids don't match
  return failure 'EBADUID'
    unless $self->uid == 0 || $self->uid == getpwnam($pidinfo->{USER});

  # Fail if commands don't match
  if (my $cmd = $self->command) {
    return failure 'EBADCOMMAND' unless $pidinfo->{COMMAND} =~ qr/^$cmd/;
  }

  # Checks all pass - kill the sucker
  $self->log->warn(sprintf 'sending kill to %s %s', $self->pid, 
    $self->command || '');
  for my $i ( 1 .. 6 ) {
    my $sig = $i < 4 ? 15 : 9;
    kill $sig, $self->pid;
    sleep 3;
    last unless $ps->is_pid_running( $self->pid );
  }

  if ($ps->is_pid_running( $self->pid )) {
    return failure 'EKILLFAILED';
  }
  else {
    return success;
  }
}

# Public interface - kill the process associated with this node
sub kill
{
  my $self = shift;

  my $rv = $self->_kill;

  # _kill seems to generate hordes of ps zombies - try doing a wait here
  $self->ps->non_blocking_wait;

  # Should only really delete/clean up if we're sure the process is killed (or otherwise gone)
  if ($rv || $rv ne 'EKILLFAILED') {
    unlink $self->file or die 'unlink of file ' . $self->file . ' failed: $!';
  }

  return $rv;
}

sub short
{
  my $self = shift;
  my @str = ( $self->time, $self->pid );
  push @str, $self->command if $self->command;
  join(', ', @str);
}

sub BUILD
{
  my $self = shift;

  # Do a basic sanity check up front - pid must at least exist
  # We don't do uid and command checks because a common use case is a (perhaps setuid) exec
  croak 'ENOPID' unless $self->fake || $self->ps->is_pid_running( $self->pid );
}


1;

__END__

=head1 NAME

Reaper::Node - class modelling a process to be reaped/terminated at a point in time

=head1 SYNOPSIS

    $node = Reaper::Node->new( 
        file    => "/var/lib/reaper/$ME.1234     
        time    => '18:00',
        pid     => 5678,
        command => 'magic_daemon_1',
        uid     => 'nobody',
    );


=head1 AUTHOR

Gavin Carr <gavin@openfusion.com.au>

=head1 COPYRIGHT

Copyright 2008 Gavin Carr.

This is free software licensed under the terms of the Perl Artistic Licence.

=cut

==> reaper-0.002000.tar.gz: reaper-0.002000/lib/Reaper/Node.pm <==
# a reaper item - a process to be terminated at a certain time

package Reaper::Node;

use Moose;
use File::stat;
use DateTime;
use DateTime::TimeZone::Tzfile;
use Unix::PID;
use Return::Value;
use Log::Log4perl qw(get_logger);
use Carp;

has 'file'          => (is => 'ro', required => 1);
has 'time'          => (is => 'ro', required => 1);
has 'tz'            => (is => 'ro', default => sub {
  DateTime::TimeZone::Tzfile->new('/etc/localtime');
});
has 'today'         => (is => 'ro', isa => 'HashRef', auto_deref => 1, lazy => 1, default => sub {
  my $self = shift;
  my $now = DateTime->now(time_zone => $self->tz);
  return { year => $now->year, month => $now->month, day => $now->day, time_zone => $self->tz };
});
has 'date'          => (is => 'ro', lazy => 1, default => sub { 
  my $self = shift;
  my ($hh, $mi) = split /:/, $self->time;
  DateTime->new( $self->today, hour => $hh, minute => $mi );
});
has 'pid'           => (is => 'ro', required => 1);
has 'command'       => (is => 'ro', default => '');
has 'uid'           => (is => 'ro', lazy => 1, default => sub { stat(shift->file)->uid });
has 'log'           => (is => 'ro', lazy => 1, default => sub { get_logger('') });
has 'ps'            => (is => 'ro', lazy => 1, default => sub { 
  Unix::PID->new or die "Cannot instantiate Unix::PID: $!";
});
# Flag for testing to indicate this is a fake process
has 'fake'          => (is => 'ro', isa => 'Bool');
# Flag for running in noop mode - don't really issue kills etc.
has 'noop'          => (is => 'ro', isa => 'Bool');

# Return the number of seconds until 'time'
sub time_seconds
{
  my $self = shift;
  my $now = shift || DateTime->now;
  return 0 if $now >= $self->date;
  return $now->subtract_datetime_absolute( $self->date )->seconds;
}

# Internal version - kill the process associated with this node
sub _kill
{
  my $self = shift;

  # Always return success for fake processes or in noop mode
  return success if $self->fake || $self->noop;

  # Fail if no process has this pid
  my $ps = $self->ps;
  return failure 'ENOPID'
    if ! $ps->is_pid_running( $self->pid );

  # Load pid info
  my $pidinfo = $ps->pid_info_hash( $self->pid )
    or die "Cannot get pid info for pid " . $self->pid;

  # Fail if uids don't match
  return failure 'EBADUID'
    unless $self->uid == 0 || $self->uid == getpwnam($pidinfo->{USER});

  # Fail if commands don't match
  if (my $cmd = $self->command) {
    return failure 'EBADCOMMAND' unless $pidinfo->{COMMAND} =~ qr/^$cmd/;
  }

  # Checks all pass - kill the sucker
  $self->log->warn(sprintf 'sending kill to %s %s', $self->pid, 
    $self->command || '');
  for my $i ( 1 .. 6 ) {
    my $sig = $i < 4 ? 15 : 9;
    $self->log->debug("sending sig $sig to pid " . $self->pid);
    kill $sig, $self->pid;
    sleep 3;
    last if ! $ps->is_pid_running( $self->pid );
  }

  if ($ps->is_pid_running( $self->pid )) {
    return failure 'EKILLFAILED';
  }
  else {
    return success;
  }
}

# Public interface - kill the process associated with this node
sub kill
{
  my $self = shift;

  my $rv = $self->_kill;

  # _kill seems to generate hordes of ps zombies - try doing a wait here
  $self->ps->non_blocking_wait;

  # Should only really delete/clean up if we're sure the process is killed (or otherwise gone)
  if ($rv || $rv ne 'EKILLFAILED') {
    unlink $self->file or die 'unlink of file ' . $self->file . ' failed: $!';
  }

  return $rv;
}

sub short
{
  my $self = shift;
  my @str = ( $self->time, $self->pid );
  push @str, $self->command if $self->command;
  join(', ', @str);
}

sub BUILD
{
  my $self = shift;

  # Do a basic sanity check up front - pid must at least exist
  # We don't do uid and command checks because a common use case is a (perhaps setuid) exec
  croak 'ENOPID' unless $self->fake || $self->ps->is_pid_running( $self->pid );
}


1;

__END__

=head1 NAME

Reaper::Node - class modelling a process to be reaped/terminated at a point in time

=head1 SYNOPSIS

    $node = Reaper::Node->new( 
        file    => "/var/lib/reaper/$ME.1234     
        time    => '18:00',
        pid     => 5678,
        command => 'magic_daemon_1',
        uid     => 'nobody',
    );


=head1 AUTHOR

Gavin Carr <gavin@openfusion.com.au>

=head1 COPYRIGHT

Copyright 2008 Gavin Carr.

This is free software licensed under the terms of the Perl Artistic Licence.

=cut
